import { alias, int, mysqlTable, text, timestamp, varchar } from 'drizzle-orm/mysql-core';
import { strike } from './strike.ts';
import type { MySql2Database } from 'drizzle-orm/mysql2';
import { and, eq } from 'drizzle-orm';
import { reportStatus } from './reportStatus.ts';
import { generateRandomString } from '@util/random.ts';
import { team } from '@db/schema/team.ts';

type Database = MySql2Database<{ report: typeof report }>;

export const report = mysqlTable('report', {
  id: int('id').primaryKey().autoincrement(),
  reason: varchar('reason', { length: 255 }).notNull(),
  body: text('body'),
  urlHash: varchar('url_hash', { length: 255 }).notNull(),
  createdAt: timestamp('created_at', { mode: 'string' }),
  reporterTeamId: int('reporter_team_id')
    .notNull()
    .references(() => team.id),
  reportedTeamId: int('reported_team_id').references(() => team.id)
});

export type AddReportReq = {
  reason: string;
  body: string | null;
  createdAt?: string | null;
  reporterTeamId: number;
  reportedTeamId?: number | null;
};

export type GetReportsReq = {
  reporter?: string | null;
  reported?: string | null;
};

export async function addReport(db: Database, values: AddReportReq) {
  const r = await db
    .insert(report)
    .values({
      reason: values.reason,
      body: values.body,
      urlHash: generateRandomString(16),
      createdAt: values.createdAt,
      reporterTeamId: values.reporterTeamId,
      reportedTeamId: values.reportedTeamId
    })
    .$returningId();

  return r[0];
}

export async function getReports(db: Database, values: GetReportsReq) {
  const reporterTeam = alias(team, 'reporter');
  const reportedTeam = alias(team, 'reported');

  let reporterIdSubquery;
  if (values.reporter != null) {
    reporterIdSubquery = db
      .select({ id: reporterTeam.id })
      .from(reporterTeam)
      .where(eq(reporterTeam.name, values.reporter))
      .as('reporter_id_subquery');
  }
  let reportedIdSubquery;
  if (values.reported != null) {
    reportedIdSubquery = db
      .select({ id: reportedTeam.id })
      .from(reportedTeam)
      .where(eq(reportedTeam.name, values.reported))
      .as('reported_id_subquery');
  }

  return db
    .select({
      id: report.id,
      reason: report.reason,
      body: report.body,
      createdAt: report.createdAt,
      reporter: {
        id: reporterTeam.id,
        name: reporterTeam.name
      },
      reported: {
        id: reportedTeam.id,
        name: reportedTeam.name
      },
      status: {
        status: reportStatus.status,
        notice: reportStatus.notice,
        statement: reportStatus.statement
      }
    })
    .from(report)
    .innerJoin(reporterTeam, eq(report.reporterTeamId, reporterTeam.id))
    .leftJoin(reportedTeam, eq(report.reportedTeamId, reportedTeam.id))
    .leftJoin(reportStatus, eq(report.id, reportStatus.reportId))
    .leftJoin(strike, eq(reportStatus.strikeId, strike.id))
    .where(
      and(
        values.reporter != null ? eq(report.reporterTeamId, reporterIdSubquery!.id) : undefined,
        values.reported != null ? eq(report.reportedTeamId, reportedIdSubquery!.id) : undefined
      )
    );
}
